/*******************************************************************************
*
* FILE:         crypton.h
*
* DESCRIPTION:  header file for crypton.c and cryptonref.c
*
* AUTHOR:       Chae Hoon Lim and Hyo Sun Hwang
*               E-mail: chlim@future.co.kr, hyosun@future.co.kr
*               http://crypt.future.co.kr/~chlim
*               Information and Communications Research Center,
*               Future Systems, Inc.
*               372-2, Yang Jae-Dong, Seo Cho-Gu, Seoul, 137-130, KOREA
*               Tel: +82-2-578-0581 (ext. 557), Fax: +82-2-578-0584
*
* DATE:         Dec. 22, 1998
*
*******************************************************************************/
#ifndef CRYPTON_H
#define CRYPTON_H

/********************** Include files ************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/********************* Type Definitions **********************/
#ifndef TYPE_DEFINITION
    #define TYPE_DEFINITION
    #if defined(__alpha)
        typedef unsigned int        DWORD;
        typedef unsigned short      WORD;
    #else
        typedef unsigned long int   DWORD;
        typedef unsigned short int  WORD;
    #endif
    typedef unsigned char           BYTE;
#endif

/******************* Constant Definitions *********************/
#define NO_ROUNDS            12
#define NO_EXP_KEYS           8
#define NO_ROUND_KEYS    ((NO_ROUNDS+1)*4)
#define BLOCK_SIZE           16    /* in bytes */
#define BLOCK_LEN           128    /* in bits */
#define CRYPTON_KEY_SIZE     32    /* in bytes */

/****************** Processor Endianness **********************/
#if !defined( LITTLE_ENDIAN ) && !defined( BIG_ENDIAN )
    #if defined( _M_I86 ) || defined( _M_IX86 ) || defined(__alpha)
        #define LITTLE_ENDIAN
    #elif defined( AMIGA )
        #define BIG_ENDIAN
    #else
        #error "Cannot determine processor endianness: define explicitly"
    #endif
#endif

/********************** Common Macros ************************/
#if defined(_MSC_VER)
    #define ROTL(x, n)     (_lrotl((x), (n)))
    #define ROTR(x, n)     (_lrotr((x), (n)))
#else
    #define ROTL(x, n)     (((x) << (n)) | ((x) >> (32-(n))))
    #define ROTR(x, n)     (((x) >> (n)) | ((x) << (32-(n))))
#endif

#define BlockCopy(pbd, pbs) {                                           \
    ((DWORD *)(pbd))[0] = ((DWORD *)(pbs))[0];                          \
    ((DWORD *)(pbd))[1] = ((DWORD *)(pbs))[1];                          \
    ((DWORD *)(pbd))[2] = ((DWORD *)(pbs))[2];                          \
    ((DWORD *)(pbd))[3] = ((DWORD *)(pbs))[3];                          \
}

#define BlockXor(pbd, pbs1, pbs2) {                                     \
    ((DWORD *)(pbd))[0] = ((DWORD *)(pbs1))[0] ^ ((DWORD *)(pbs2))[0];  \
    ((DWORD *)(pbd))[1] = ((DWORD *)(pbs1))[1] ^ ((DWORD *)(pbs2))[1];  \
    ((DWORD *)(pbd))[2] = ((DWORD *)(pbs1))[2] ^ ((DWORD *)(pbs2))[2];  \
    ((DWORD *)(pbd))[3] = ((DWORD *)(pbs1))[3] ^ ((DWORD *)(pbs2))[3];  \
}

#define BlockSet(pbd, pdwValue) {       \
    ((DWORD *)(pbd))[0] = pdwValue;     \
    ((DWORD *)(pbd))[1] = pdwValue;     \
    ((DWORD *)(pbd))[2] = pdwValue;     \
    ((DWORD *)(pbd))[3] = pdwValue;     \
}

/************** Function Prototype Declarations **************/
#ifndef PROTOTYPES
#define PROTOTYPES  1
#endif

#if PROTOTYPES
	#define PROTO_LIST(list)    list
#else
	#define PROTO_LIST(list)    ()
#endif

void CryptonEncrypt PROTO_LIST((BYTE *pbData, DWORD *pdwRoundKey));

#define CryptonDecrypt(pbData, pdwRoundKey) CryptonEncrypt(pbData, pdwRoundKey)

void CryptonExpandKey
    PROTO_LIST((DWORD *pdwExpandedKey, BYTE * pbUserKey, int dwUserKeyLen));
void CryptonGenEncRoundKey
    PROTO_LIST((DWORD *pdwEncRoundKey, DWORD *pdwExpandedKey));
void CryptonGenDecRoundKey
    PROTO_LIST((DWORD *pdwDecRoundKey, DWORD *pdwExpandedKey));
void CryptonGenSbox(void);

/******************************************************************/
#endif
